@@ -23,14 +23,13 @@ from thrift.protocol import TProtocol
 from thrift.Thrift import TType
 
 from concrete import Communication
+from concrete.util import read_communication_from_file
 
 
 
 def validate_communication_file(communication_filename):
     logging.info(ilm(0, "Opening Concrete Communication with filename '%s'" % communication_filename))
-    comm = Communication()
-    comm_bytestring = open(communication_filename).read()
-    TSerialization.deserialize(comm, comm_bytestring)
+    comm = read_communication_from_file(communication_filename)
     validate_communication(comm)
 
 
@@ -56,8 +55,10 @@ def validate_communication(comm):
                                   (section.uuid, len(section.sentenceList))))
                 for sentence in section.sentenceList:
                     valid &= validate_token_offsets_for_sentence(sentence)
-                    valid &= validate_constituency_parses(comm, sentence.tokenization)
-                    valid &= validate_dependency_parses(sentence.tokenization)
+                    if sentence.tokenization:
+                        valid &= validate_constituency_parses(comm, sentence.tokenization)
+                        valid &= validate_dependency_parses(sentence.tokenization)
+                        valid &= validate_token_taggings(sentence.tokenization)
 
     valid &= validate_entity_mention_ids(comm)
     valid &= validate_entity_mention_tokenization_ids(comm)
@@ -267,7 +268,7 @@ def validate_constituency_parses(comm, tokenization):
             if not nx.is_connected(undirected_graph):
                 valid = False
                 logging.error(ilm(6, "The constituent parse \"tree\" is not a fully connected graph - the graph has %d components" %
-                    len(nx.connected_components(undirected_graph))))
+                    nx.number_connected_components(undirected_graph)))
 
             # Check if constituent parse "tree" is actually a tree
             if nx.number_of_nodes(constituent_parse_tree) != nx.number_of_edges(constituent_parse_tree) + 1:
@@ -289,6 +290,7 @@ def validate_dependency_parses(tokenization):
     valid = True
 
     if tokenization.dependencyParseList:
+        total_tokens = len(tokenization.tokenList.tokenList)
         for dependencyParse in tokenization.dependencyParseList:
             dependency_parse_tree = nx.DiGraph()
 
@@ -296,9 +298,13 @@ def validate_dependency_parses(tokenization):
             for dependency in dependencyParse.dependencyList:
                 if dependency.gov is None and dependency.edgeType.lower() != "root":
                     valid = False
-                    logging.error(ilm(6, "Found a null dependency parse node with governer whose edgeType is '%s' instead of 'root'" %
+                    logging.error(ilm(7, "Found a null dependency parse node with governer whose edgeType is '%s' instead of 'root'" %
                                           dependency.edgeType))
                 if dependency.gov is not None:
+                    if dependency.gov < -1 or dependency.gov > total_tokens:
+                        valid = False
+                        logging.error(ilm(7, "Found a null dependency parse node with invalid governer of '%d'" %
+                                          dependency.gov))
                     dependency_parse_tree.add_node(dependency.gov)
                 dependency_parse_tree.add_node(dependency.dep)
 
@@ -312,10 +318,10 @@ def validate_dependency_parses(tokenization):
             try:
                 if not nx.is_connected(undirected_graph):
                     valid = False
-                    logging.error(ilm(6, "The dependency parse \"tree\" is not a fully connected graph - the graph has %d components" %
-                                      len(nx.connected_components(undirected_graph))))
+                    logging.error(ilm(7, "The dependency parse \"tree\" is not a fully connected graph - the graph has %d components" %
+                                      nx.number_connected_components(undirected_graph)))
             except nx.exception.NetworkXPointlessConcept:
-                logging.warning(ilm(6, "The dependency parse \"tree\" does not have any nodes"))
+                logging.warning(ilm(7, "The dependency parse \"tree\" does not have any nodes"))
     return valid
 
 
@@ -523,6 +529,22 @@ def validate_token_ref_sequence(comm, token_ref_sequence):
     return valid
 
 
+def validate_token_taggings(tokenization):
+    """
+    Test if a Tokenization has any TokenTaggings with invalid token indices
+    """
+    valid = True
+    if tokenization and tokenization.tokenTaggingList:
+        total_tokens = len(tokenization.tokenList.tokenList)
+        for token_tagging in tokenization.tokenTaggingList:
+            for tagged_token in token_tagging.taggedTokenList:
+                if tagged_token.tokenIndex >= total_tokens or tagged_token.tokenIndex < 0:
+                    valid = False
+                    logging.error(ilm(7, "TokenTagging '%s' has a tokenIndex '%d' that is out of bounds." %
+                                      (token_tagging.uuid.uuidString, tagged_token.tokenIndex)))
+    return valid
+
+
 def validate_thrift_object_required_fields(thrift_object, indent_level=0):
     """
     Test if a thrift object has all required fields.
